import os
import argparse
from loguru import logger
from omegaconf import OmegaConf
from omegaconf.dictconfig import DictConfig
from omegaconf.listconfig import ListConfig


def read_yaml_cfg_impl(config_file: str) -> DictConfig:
    """
    Parse yaml config file, and merge sub-yaml file within the `include` filed.
    """
    conf = OmegaConf.load(config_file)
    if conf.get("include") is not None:
        if isinstance(conf.include, str):
            include_files = [conf.include]
        elif isinstance(conf.include, ListConfig):
            include_files = conf.include
        else:
            raise TypeError("Only str/ListConfig are allowed for `include` field.")
        for include_file in include_files:
            if include_file.startswith("."):
                # Interpolate the full path.
                cur_config_path = os.path.dirname(os.path.abspath(config_file))
                include_file_full_path = f"{cur_config_path}/{include_file}"
                sub_conf = read_yaml_cfg_impl(include_file_full_path)
            else:
                sub_conf = read_yaml_cfg_impl(include_file)
            conf = OmegaConf.merge(sub_conf, conf)
        conf.pop("include", None)
    return conf


def read_yaml_cfg(config_file: str, verbose=True) -> DictConfig:
    cfg = read_yaml_cfg_impl(config_file)
    if verbose:
        logger.debug(f"Config: \n{OmegaConf.to_yaml(cfg)}")
    return cfg


def paser_yaml_cfg():
    parser = argparse.ArgumentParser(prog='llm')
    parser.add_argument('-c', '--config_file', type=str, help="The config file path")
    args = parser.parse_args()
    assert os.path.exists(args.config_file), f"yaml config file not exist {args.config_file}"
    return read_yaml_cfg(args.config_file)



from typing import Any

from omegaconf.dictconfig import DictConfig


class Registry:
    """
    Registry for any modular functions or methods.
    """

    def __init__(self, name: str) -> None:
        self._dict = {}
        self._name = name

    def __len__(self) -> int:
        return len(self._dict)

    def __getitem__(self, module_name: str) -> Any:
        if module_name in self._dict:
            return self._dict[module_name]
        else:
            raise KeyError(f"{module_name} is not registerd in {self._name} yet.")

    def __setitem__(self, module_name: str, module: Any) -> None:
        if module_name in self._dict:
            raise KeyError(f"{module_name} has already been registered in {self._name}.")
        self._dict[module_name] = module

    def __contains__(self, module_name: str):
        return self.get(module_name) is not None

    def __repr__(self) -> str:
        msg = "Registered modules:\n"
        msg += ', '.join(list(self._dict.keys()))
        return msg

    @property
    def name(self) -> str:
        return self._name

    def _register_module(self, module: Any) -> Any:
        module_name = module.__name__
        self[module_name] = module
        return module

    def register(self, module=None) -> Any:
        if module is not None:
            return self._register_module(module)

        def _register(module):
            return self._register_module(module)

        return _register

    def build(self, cfg: DictConfig, **kwargs) -> Any:
        if cfg.get("type") is None:
            raise TypeError("cfg must have 'type' for building class.")
        cfg_args = cfg.copy()
        obj_name = cfg_args.pop("type")
        func_args = {**kwargs}
        func_args.update(cfg_args)
        return self[obj_name](**func_args)



import os
from setuptools import setup, find_packages

VERSION = '0.0.1'
DESCRIPTION = 'Utility library for deep learning'


def main():
    cwd = os.path.dirname(os.path.abspath(__file__))
    INSTALL_REQUIRES = []
    DEPENDENCY_LINKS = []

    # with open(f"{cwd}/requirements.txt", "r") as file:
    with open("./requirements.txt", "r") as file:
        for line in file.readlines():
            line = line.strip()
            if (not line) or line.startswith("#"):
                continue
            if line.startswith("git+"):
                DEPENDENCY_LINKS.append(line)
                INSTALL_REQUIRES.append(line.split("#egg=")[1])
            else:
                INSTALL_REQUIRES.append(line)

        with open("./README.md", encoding="utf-8") as f:
            LONG_DESCRIPTION = f.read()
    excludes = ['output', 'artifacts']

    setup(
        name="pydlutils",
        version=VERSION,
        author="zebincai",
        author_email="1028798080@qq.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        packages=find_packages(exclude=excludes),
        url="https://github.com/zebincai/pydlutils",
        download_url="https://github.com/zebincai/pydlutils/tags",
        install_requires=INSTALL_REQUIRES,  # add any additional packages that
        dependency_links=DEPENDENCY_LINKS,
        include_package_data=True,
        keywords=['python', 'compute vision', 'deep learning', 'LLM'],
        license='Apache',
        classifiers=[
            "Development Status :: 1 - Planning",
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'Intended Audience :: Science/Research',
            'Topic :: Software Development',
            'Topic :: Software Development :: Libraries',
            'Topic :: Software Development :: Libraries :: Python Modules',
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
        ])


if __name__ == "__main__":
    main()
